@@ -9,9 +9,12 @@ from __future__ import annotations
 
 from typing import Any, List, Optional
 
+from concepttordf import Concept
 from datacatalogtordf import Agent, Resource, URI
 from rdflib import BNode, Graph, Literal, Namespace, RDF, URIRef
 
+from modelldcatnotordf.licensedocument import LicenseDocument
+
 DCT = Namespace("http://purl.org/dc/terms/")
 DCAT = Namespace("http://www.w3.org/ns/dcat#")
 ODRL = Namespace("http://www.w3.org/ns/odrl/2/")
@@ -34,13 +37,15 @@ class InformationModel(Resource):
         "_subject",
         "_modelelements",
         "_informationmodelidentifier",
+        "_licensedocument",
     )
 
     _title: dict
     _publisher: Agent
-    _subject: List[str]
+    _subject: List[Concept]
     _modelelements: List[ModelElement]
     _informationmodelidentifier: str
+    _licensedocument: LicenseDocument
 
     def __init__(self) -> None:
         """Inits InformationModel object with default values."""
@@ -51,7 +56,7 @@ class InformationModel(Resource):
 
     @property
     def informationmodelidentifier(self) -> str:
-        """Get/set for informationmodelidentifier."""
+        """Get for informationmodelidentifier."""
         return self._informationmodelidentifier
 
     @informationmodelidentifier.setter
@@ -63,9 +68,18 @@ class InformationModel(Resource):
         """Get for type."""
         return self._type
 
+    @property
+    def licensedocument(self) -> LicenseDocument:
+        """Get for license."""
+        return self._licensedocument
+
+    @licensedocument.setter
+    def licensedocument(self, licensedocument: LicenseDocument) -> None:
+        self._licensedocument = licensedocument
+
     @property
     def title(self) -> dict:
-        """Get/set for title."""
+        """Get for title."""
         return self._title
 
     @title.setter
@@ -74,7 +88,7 @@ class InformationModel(Resource):
 
     @property
     def description(self) -> dict:
-        """Get/set for description."""
+        """Get for description."""
         return self._description
 
     @description.setter
@@ -83,7 +97,7 @@ class InformationModel(Resource):
 
     @property
     def theme(self) -> List[str]:
-        """Get/set for theme."""
+        """Get for theme."""
         return self._theme
 
     @theme.setter
@@ -92,7 +106,7 @@ class InformationModel(Resource):
 
     @property
     def publisher(self: InformationModel) -> Agent:
-        """Get/set for publisher."""
+        """Get for publisher."""
         return self._publisher
 
     @publisher.setter
@@ -100,13 +114,13 @@ class InformationModel(Resource):
         self._publisher = publisher
 
     @property
-    def subject(self: InformationModel) -> List[str]:
-        """Get/set for subject."""
+    def subject(self: InformationModel) -> List[Concept]:
+        """Get for subject."""
         return self._subject
 
     @property
     def modelelements(self: InformationModel) -> List[ModelElement]:
-        """Get/set for modelelements."""
+        """Get for modelelements."""
         return self._modelelements
 
     def to_rdf(
@@ -141,6 +155,7 @@ class InformationModel(Resource):
         self._publisher_to_graph()
         self._subject_to_graph()
         self._modelelements_to_graph()
+        self._licensedocument_to_graph()
 
         if getattr(self, "informationmodelidentifier", None):
             self._g.add(
@@ -155,8 +170,15 @@ class InformationModel(Resource):
 
     def _subject_to_graph(self: InformationModel) -> None:
         if getattr(self, "subject", None):
+
             for subject in self._subject:
-                self._g.add((URIRef(self.identifier), DCT.subject, URIRef(subject)))
+
+                _subject = URIRef(subject.identifier)
+
+                for _s, p, o in subject._to_graph().triples((None, None, None)):
+                    self._g.add((_subject, p, o))
+
+                self._g.add((URIRef(self.identifier), DCT.subject, _subject,))
 
     def _modelelements_to_graph(self: InformationModel) -> None:
 
@@ -180,6 +202,23 @@ class InformationModel(Resource):
                     )
                 )
 
+    def _licensedocument_to_graph(self: InformationModel) -> None:
+
+        if getattr(self, "licensedocument", None):
+
+            if getattr(self.licensedocument, "identifier", None):
+                _licensedocument = URIRef(self.licensedocument.identifier)
+
+            else:
+                _licensedocument = BNode()
+
+            for _s, p, o in self.licensedocument._to_graph().triples(
+                (None, None, None)
+            ):
+                self._g.add((_licensedocument, p, o))
+
+            self._g.add((URIRef(self.identifier), DCT.license, _licensedocument))
+
 
 class ModelElement:
     """A class representing a modelldcatno:ModelElement."""
@@ -199,7 +238,7 @@ class ModelElement:
     _identifier: URI
     _dct_identifier: str
     _has_property: List[Any]
-    _subject: str
+    _subject: Concept
 
     def __init__(self) -> None:
         """Inits an object with default values."""
@@ -208,7 +247,7 @@ class ModelElement:
 
     @property
     def identifier(self) -> str:
-        """Get/set for identifier."""
+        """Get for identifier."""
         return self._identifier
 
     @identifier.setter
@@ -217,7 +256,7 @@ class ModelElement:
 
     @property
     def dct_identifier(self) -> str:
-        """Get/set for dct_identifier."""
+        """Get for dct_identifier."""
         return self._dct_identifier
 
     @dct_identifier.setter
@@ -234,17 +273,17 @@ class ModelElement:
         self._title = title
 
     @property
-    def subject(self) -> str:
-        """Get/set for subject."""
+    def subject(self) -> Concept:
+        """Get for subject."""
         return self._subject
 
     @subject.setter
-    def subject(self, subject: str) -> None:
+    def subject(self, subject: Concept) -> None:
         self._subject = subject
 
     @property
     def has_property(self) -> List[Any]:
-        """Get/set for has_type."""
+        """Get for has_type."""
         return self._has_property
 
     def to_rdf(self, format: str = "turtle", encoding: Optional[str] = "utf-8") -> str:
@@ -259,9 +298,12 @@ class ModelElement:
         """
         return self._to_graph().serialize(format=format, encoding=encoding)
 
-    def _to_graph(self) -> Graph:
+    def _to_graph(self, type: str = MODELLDCATNO.ModelElement) -> Graph:
         """Returns the modelelement as graph.
 
+         Args:
+            type: type for identifying class. Default: MODELLDCATNO.ModelElement
+
         Returns:
             the modelelement graph
         """
@@ -277,7 +319,7 @@ class ModelElement:
         else:
             _self = BNode()
 
-        self._g.add((_self, RDF.type, MODELLDCATNO.ModelElement))
+        self._g.add((_self, RDF.type, type))
 
         if getattr(self, "title", None):
             for key in self.title:
@@ -287,7 +329,13 @@ class ModelElement:
             self._g.add((_self, DCT.identifier, Literal(self._dct_identifier)))
 
         if getattr(self, "subject", None):
-            self._g.add((_self, DCT.subject, URIRef(self.subject)))
+
+            _subject = URIRef(self.subject.identifier)
+
+            for _s, p, o in self.subject._to_graph().triples((None, None, None)):
+                self._g.add((_subject, p, o))
+
+            self._g.add((_self, DCT.subject, _subject))
 
         if getattr(self, "has_property", None):
             self._has_property_to_graph(_self)
@@ -330,7 +378,7 @@ class ModelProperty:
     _min_occurs: int
     _max_occurs: int
     _title: dict
-    _subject: str
+    _subject: Concept
 
     def __init__(self) -> None:
         """Inits an object with default values."""
@@ -338,12 +386,12 @@ class ModelProperty:
         self._has_type = []
 
     @property
-    def subject(self) -> str:
-        """Get/set for subject."""
+    def subject(self) -> Concept:
+        """Get for subject."""
         return self._subject
 
     @subject.setter
-    def subject(self, subject: str) -> None:
+    def subject(self, subject: Concept) -> None:
         self._subject = subject
 
     @property
@@ -357,12 +405,12 @@ class ModelProperty:
 
     @property
     def has_type(self) -> List[ModelElement]:
-        """Get/set for has_type."""
+        """Get for has_type."""
         return self._has_type
 
     @property
     def identifier(self) -> str:
-        """Get/set for identifier."""
+        """Get for identifier."""
         return self._identifier
 
     @identifier.setter
@@ -371,7 +419,7 @@ class ModelProperty:
 
     @property
     def min_occurs(self) -> int:
-        """Get/set for min_occurs."""
+        """Get for min_occurs."""
         return self._min_occurs
 
     @min_occurs.setter
@@ -380,7 +428,7 @@ class ModelProperty:
 
     @property
     def max_occurs(self) -> int:
-        """Get/set for max_occurs."""
+        """Get for max_occurs."""
         return self._max_occurs
 
     @max_occurs.setter
@@ -399,9 +447,15 @@ class ModelProperty:
         """
         return self._to_graph().serialize(format=format, encoding=encoding)
 
-    def _to_graph(self) -> Graph:
+    def _to_graph(
+        self, type: str = MODELLDCATNO.Property, selfobject: Any = None
+    ) -> Graph:
         """Returns the property as graph.
 
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.Property
+            selfobject: a bnode or URI passed from an subclass Default: None
+
         Returns:
             the property graph
         """
@@ -411,12 +465,16 @@ class ModelProperty:
         self._g.bind("dct", DCT)
         self._g.bind("skos", SKOS)
 
-        if getattr(self, "identifier", None):
+        if selfobject:
+            _self = selfobject
+
+        elif getattr(self, "identifier", None):
             _self = URIRef(self.identifier)
+
         else:
             _self = BNode()
 
-        self._g.add((_self, RDF.type, MODELLDCATNO.Property))
+        self._g.add((_self, RDF.type, type))
 
         self._has_type_to_graph(_self)
 
@@ -431,7 +489,13 @@ class ModelProperty:
                 self._g.add((_self, DCT.title, Literal(self.title[key], lang=key),))
 
         if getattr(self, "subject", None):
-            self._g.add((_self, DCT.subject, URIRef(self.subject)))
+
+            _subject = URIRef(self.subject.identifier)
+
+            for _s, p, o in self.subject._to_graph().triples((None, None, None)):
+                self._g.add((_subject, p, o))
+
+            self._g.add((_self, DCT.subject, _subject))
 
         return self._g
 
@@ -449,3 +513,121 @@ class ModelProperty:
                     self._g.add((_has_type, p, o))
 
                 self._g.add((_self, MODELLDCATNO.hasType, _has_type,))
+
+
+class Role(ModelProperty):
+    """A class representing a modelldcatno:Role."""
+
+    __slots__ = (
+        "_identifier",
+        "_has_object_type",
+    )
+
+    _identifier: URI
+    _has_object_type: ObjectType
+    _g: Graph
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        super().__init__()
+
+    @property
+    def has_object_type(self: Role) -> ObjectType:
+        """Get for has_object_type."""
+        return self._has_object_type
+
+    @has_object_type.setter
+    def has_object_type(self: Role, has_object_type: Any) -> None:
+        self._has_object_type = has_object_type
+
+    def to_rdf(
+        self: Role, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the role to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(
+        self: Role, type: str = MODELLDCATNO.Role, selfobject: Any = None
+    ) -> Graph:
+        """Returns the role as graph.
+
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.Role
+            selfobject: a bnode or URI passed from an subclass Default: None
+
+        Returns:
+            the role graph
+        """
+        if getattr(self, "identifier", None):
+            _self = URIRef(self.identifier)
+        else:
+            _self = BNode()
+
+        super(Role, self)._to_graph(MODELLDCATNO.Role, _self)
+
+        self._has_object_type_to_graph(_self)
+
+        return self._g
+
+    def _has_object_type_to_graph(self, _self: Any) -> None:
+
+        if getattr(self, "has_object_type", None):
+
+            if getattr(self._has_object_type, "identifier", None):
+                _has_object_type = URIRef(self._has_object_type.identifier)
+            else:
+                _has_object_type = BNode()
+
+            for _s, p, o in self._has_object_type._to_graph().triples(
+                (None, None, None)
+            ):
+                self._g.add((_has_object_type, p, o))
+
+            self._g.add((_self, MODELLDCATNO.hasObjectType, _has_object_type))
+
+
+class ObjectType(ModelElement):
+    """A class representing a modelldcatno:ObjectType."""
+
+    _identifier: URI
+    _dct_identifier: str
+    _g: Graph
+
+    def __init__(self) -> None:
+        """Inits an object with default values."""
+        super().__init__()
+
+    def to_rdf(
+        self: ObjectType, format: str = "turtle", encoding: Optional[str] = "utf-8"
+    ) -> str:
+        """Maps the object type to rdf.
+
+        Args:
+            format: a valid format. Default: turtle
+            encoding: the encoding to serialize into
+
+        Returns:
+            a rdf serialization as a string according to format.
+        """
+        return self._to_graph().serialize(format=format, encoding=encoding)
+
+    def _to_graph(self: ObjectType, type: str = MODELLDCATNO.ObjectType) -> Graph:
+        """Returns the object type as graph.
+
+        Args:
+            type: type for identifying class. Default: MODELLDCATNO.ObjectType
+
+        Returns:
+            the object type graph
+        """
+        super(ObjectType, self)._to_graph(MODELLDCATNO.ObjectType)
+
+        return self._g
